;;;
;;; scbib - a bibliography management system written in Scheme
;;;
;;; Copyright (C) 2004 Satoru Takabayashi <satoru@namazu.org> 
;;;     All rights reserved.
;;;     This is free software with ABSOLUTELY NO WARRANTY.
;;;
;;; You can redistribute it and/or modify it under the terms of 
;;; the Gauche's licence.
;;;
(define-module scbib.bibtex
  (use gauche.regexp)
  (use scbib)
  (export scbib-bibtex-print
          ))
(select-module scbib.bibtex)

(define (scbib-bibtex-print bibdb bibitem)
  (define (scbib-bibtex-print-sub bibitem)
    (define (authors)
      (let ((authors (scbib-get-authors bibitem)))
	(cond ((eq? authors #f)
	       "")
	      ((= (length authors) 1)
	       (car authors))
	      (else
	       (string-join authors " and ")))))

    (define (editors)
      (let ((editors (scbib-values bibitem 'editor)))
	(cond ((eq? editors #f)
	       #f)
	      ((= (length editors) 1)
	       (car editors))
	      (else
	       (string-join editors " and ")))))

    (define (escape string)
      (set! string (regexp-replace-all #/#/ string "\\\\#"))
      (set! string (regexp-replace-all #/~/ string "\\\\~{}"))
      (set! string (regexp-replace-all #/\/\\~\{\}/ string "\\\\slash\\\\~{}"))
      (set! string (regexp-replace-all #/\// string "{\\\\slash}"))
      string)

    (define (get field)
      (let ((item (scbib-value bibitem field)))
	(if item
	    (escape item)
	    #f)))

    (define (itemize field-name item)
      (if item
	  (format #f "    ~a = {~a},\n" field-name item)
	  ""))

    (define (note)
      (let ((x (get 'note)))
	(or x (if (equal? (get 'bibtype) "web")
		  (get 'web)
		  #f))))

    (format #t "@~a{~a,\n~a~a~a~a~a~a~a~a~a~a~a~a~a~a~a~a}\n"
	    (let ((bibtype (get 'bibtype)))
	      (if (string=? bibtype "web")
		  "misc"
		  bibtype))
	    (scbib-get-abbrev bibitem)
	    (itemize "author" (authors))
	    (itemize "editor" (editors))
	    (itemize "title" (get 'title))
	    (itemize "series" (get 'series))
	    (itemize "edition" (get 'edition))
	    (itemize "publisher" (get 'publisher))
	    (itemize "year" (get 'year))

	    (itemize "pages" (get 'pages))
	    (itemize "booktitle" (get 'booktitle))
	    (itemize "organization" (get 'organization))
	    (itemize "address" (get 'address))
	    (itemize "note" (note))
;	    (itemize "month" (get 'month))
	    (itemize "volume" (get 'volume))
	    (itemize "number" (get 'number))
	    (itemize "school" (get 'school))
	    (itemize "journal" (get 'journal)))
    #f)

  (scbib-bibtex-print-sub bibitem))

